﻿package hype.extended.color {
	import hype.framework.color.IColorist;

	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.InteractiveObject;
	import flash.display.Sprite;
	import flash.geom.ColorTransform;

	/**
	 * Colorist that applies colors from a linked image
	 */
	public class PixelColorist implements IColorist {
		
		private var _image:Class;
		private var _imageBitmapData:BitmapData;
		private var _width:Number;
		private var _height:Number;
		
		/**
		 * Constructor
		 * 
		 * @param image Class name of Linked image in library (Class)
		 * @param width Width of Linked image in library (Number)
		 * @param height Height of Linked image in library (Number)
		 */
		public function PixelColorist(image:Class, width:Number, height:Number) {
			_image = image;
			_width = width;
			_height = height;
			
			_imageBitmapData = new _image(_width, _height);
		}

		/**
		 * Color all InteractiveObject children of a sprite with the color that corresponds to the position of the sprite over the image
		 * 
		 * @param sprite Sprite that should have it's children colored
		 */
		public function colorChildren(sprite:Sprite):void {
			var max:uint = sprite.numChildren;
			var i:uint;
			var child:DisplayObject;
			var rgb:uint;
			
			for (i=0; i<max; ++i) {
				child = sprite.getChildAt(i);
				if (child is InteractiveObject) {
					rgb = _imageBitmapData.getPixel((sprite.x + child.x), (sprite.y + child.y));
					child.transform.colorTransform = new ColorTransform(0, 0, 0, 1, rgb >> 16, rgb >> 8 & 255, rgb & 255, 0);
				}
			}
		}
		
		/**
		 * Randomly choose a color from the image
		 * 
		 * @return integer color value
		 */
		public function getColor():uint {
			var x:uint = Math.floor(Math.random() * _width);
			var y:uint = Math.floor(Math.random() * _height);
			
			return _imageBitmapData.getPixel(x, y);
		}
	}
}


